/* CablePanel.java
 *
 * Jim McBeath, March 14, 2003
 */

package net.jimmc.spelsim;

class CablePanel extends PartPanel {
    final Cable cable;
    ChoiceField cableChoice;
    DoubleField capacityField;
    DoubleField lengthField;
    DoubleField safetyFactorField;
    IntegerField finiteElementsField;
    DoubleField massField;
    DoubleField cwRatioField;
    DoubleField baseAreaField;
    DoubleField taperField;
    DoubleField cOfMassField;	//center of mass altitude
    DoubleField cOfGravityField;
    DoubleField cOfCentrifugalField;

    public CablePanel(SpelControls controls, Cable c) {
    	super(controls);
	this.cable = c;
	String[] knownCables = Cable.getCableNames();
	String[] cables = addCustom(knownCables);
	cableChoice = addChoice("Cable",cables,partFont);
	cableChoice.addChoiceListener(new ChoiceListener() {
	    public void choiceSelected(String s) {
	    	selectCable(s);
	    }
	});
	capacityField = addDoubleField("Capacity","kg");
	    //Amount of weight the cable is capable of supporting
	    //at the surface of the planet
	capacityField.addEnterListener(new EnterListener() {
	    public void valueEntered(double d) {
	        cable.setCapacity(d);
		updateCable();
	    }
	});
	lengthField = addDoubleField("Length","m");
	lengthField.addEnterListener(new EnterListener() {
	    public void valueEntered(double d) {
	        cable.setLength(d);
		updateCable();
	    }
	});
	safetyFactorField = addDoubleField("Safety Factor","");
	safetyFactorField.addEnterListener(new EnterListener() {
	    public void valueEntered(double d) {
	        cable.setSafetyFactor(d);
		updateCable();
	    }
	});
	finiteElementsField = addIntegerField("Finite Elements","");
	finiteElementsField.addEnterListener(new EnterListener() {
	    public void valueEntered(int n) {
	        cable.setNumElements(n);
		updateCable();
	    }
	});
	massField = addDoubleField("Mass","kg",readOnlyFont);
	massField.setEditable(false);
	cwRatioField = addDoubleField("CW/Mass","",readOnlyFont);
	cwRatioField.setEditable(false);
	baseAreaField = addDoubleField("Base Area","m^2",readOnlyFont);
	baseAreaField.setEditable(false);
	taperField = addDoubleField("Taper","",readOnlyFont);
	taperField.setEditable(false);
	cOfGravityField = addDoubleField("C of Gravity","m",readOnlyFont);
	cOfGravityField.setEditable(false);
	cOfMassField = addDoubleField("C of Mass","m",readOnlyFont);
	cOfMassField.setEditable(false);
	cOfCentrifugalField =
		addDoubleField("C of Centrifugal","m",readOnlyFont);
	cOfCentrifugalField.setEditable(false);

	cableChoice.select(cables[0]);
        selectCable(cableChoice.getSelectedItem());
    }

    void selectCable(String s) {
        boolean isCustom = s.equals("custom");
	capacityField.setEditable(isCustom);
	lengthField.setEditable(isCustom);
	safetyFactorField.setEditable(isCustom);
	finiteElementsField.setEditable(isCustom);
	if (isCustom)
	    return;
	Cable c = Cable.getCable(s);
	capacityField.setValue(c.getCapacity());
	lengthField.setValue(c.getLength());
	safetyFactorField.setValue(c.getSafetyFactor());
	finiteElementsField.setValue(c.getNumElements());
	updateCable();
    }

    public void updateCable() {
	cable.setCapacity(capacityField.getDoubleValue());
	cable.setLength(lengthField.getDoubleValue());
	cable.setSafetyFactor(safetyFactorField.getDoubleValue());
	cable.setNumElements(finiteElementsField.getIntValue());
	massField.setValue(cable.getMass());
	cwRatioField.setValue(cable.getCounterweightMass()/cable.getMass());
	baseAreaField.setValue(cable.getBaseArea());
	taperField.setValue(cable.getTaper());
	cOfMassField.setValue(cable.getCenterOfMassAltitude());
	cOfGravityField.setValue(cable.getCenterOfGravityAltitude());
	cOfCentrifugalField.setValue(cable.getCenterOfCentrifugalAltitude());
    }
}

//end
