/* DataSet.java
 *
 * Jim McBeath, November 16, 2007
 */

package net.jimmc.spelsim;

/** A data set for holding the data we use when plotting. */
public class DataSet {
    private String name = "NoName";
    private double[] data = new double[0];
    private double minRange = 0.0;

    public DataSet(){};

    public void setName(String name) {
        if (name!=null && !name.equals(""))
            this.name = name;
    }

    public String getName() {
        return name;
    }

    /** Set scalar data. */
    public void setData(double[] data) {
        this.data = new double[data.length];
        System.arraycopy(data,0,this.data,0,data.length);
    }

    public double[] getScalarData() {
        return this.data;       //TODO - copy?
    }

    /** Set the minimum value that getMaximumRange will return. */
    public void setMinimumRange(double r) {
        minRange = r;
    }

    /** Get the maximum magnitude of any of our data. */
    public double getMaximumMagnitude() {
        double max = minRange;
        for (int i=0; i<data.length; i++) {
            double x = data[i];
            if (x>max)
                max = x;
            else if (-x>max)    //negative numbers
                max = -x;
        }
        return max;
    }
}
