/* DoubleField.java
 *
 * Jim McBeath, March 14, 2003
 */

package net.jimmc.spelsim;

import java.awt.Color;
import java.awt.TextField;
import java.text.DecimalFormat;
import java.text.Format;

class DoubleField extends TextField {
    Format fmt;		//floating point with exponent
    Format fmtE1;
    Format fmtE2;
    Format fmtF;	//floating point without exponent
    public DoubleField() {
        super(10);
	fmt =   new DecimalFormat("0.####E0");
	fmtE1 = new DecimalFormat("00.###E0");
	fmtE2 = new DecimalFormat("000.##E0");
	fmtF =  new DecimalFormat("0.####");
    }
    public void addEnterListener(EnterListener enterListener) {
        addActionListener(enterListener);
	addFocusListener(enterListener);
    }
    public double getDoubleValue() {
        String s = getText();
	if (s==null || s.trim().equals(""))
	    return 0.0;		//nothing to parse
	return Double.parseDouble(s);
    }
    public void setValue(double d) {
        setText(getDisplayString(d));
    }
    public void setEditable(boolean editable) {
        super.setEditable(editable);
	setBackground(editable?Color.white:Color.lightGray);
    }

    /** Convert a double to a string value for display. */
    protected String getDisplayString(double d) {
    	Double dd = new Double(d);
        String s = fmt.format(dd);
	int xe = s.lastIndexOf("E");
	String expStr = s.substring(xe+1);	//get the exponent string
	int exp = Integer.parseInt(expStr);
	if (exp>=-1 && exp<=2) {
	    //Small exponent, format it as floating point with no exponent
	    return fmtF.format(dd);
	}
	if (exp<0) {
	    switch ((-exp)%3) {
	    default:
	    case 0: return s;		//use it as is
	    case 1: return fmtE2.format(dd);
	    case 2: return fmtE1.format(dd);
	    }
	} else {
	    switch (exp%3) {
	    default:
	    case 0: return s;		//use it as is
	    case 1: return fmtE1.format(dd);
	    case 2: return fmtE2.format(dd);
	    }
	}
    }
}

//end
