/* EventPanel.java
 *
 * Jim McBeath, December 19, 2007
 */

package net.jimmc.spelsim;

import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Component;
import java.awt.Container;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Label;
import java.awt.Panel;
import java.awt.ScrollPane;
import java.awt.TextArea;
import java.awt.TextField;
import java.util.EventObject;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;

/** The panel implementing the tab for the Event button. */
class EventPanel extends Panel implements TabSelectListener {

    private EventListPanel eventListPanel;
    private ScrollPane scrollPane;
    private SpelCanvas canvas;

    public EventPanel(SpelCanvas canvas) {
        super();
        this.canvas = canvas;

        eventListPanel = new EventListPanel(this);

        GridBagLayout gbl = new GridBagLayout();
        this.setLayout(gbl);
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.anchor = GridBagConstraints.NORTH;

        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.gridwidth = 1;
        gbc.weightx = 1;
        gbc.weighty = 0;

        Button addEventButton = new Button("Add Event");
        addEventButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ev) {
                eventListPanel.addEventRow();
            }
        });
        gbl.setConstraints(addEventButton,gbc);
        add(addEventButton);

        gbc.gridy++;
        gbc.weighty = 1;
        gbc.fill = GridBagConstraints.BOTH;

        scrollPane = new ScrollPane();
        scrollPane.add(eventListPanel);
        this.add(scrollPane, gbc);
    }

    public void fixLayout() {
        doLayout();
        if (scrollPane!=null)
            scrollPane.doLayout();
    }

    public void tabSelected() {
        ;
    }

    public void tabDeselected() {
        //collect the events and pass them to the simulator
        SimEvent[] simEvents = eventListPanel.collectSimEvents();
        canvas.setSimEvents(simEvents);
    }
}

class EventRow implements ActionListener {
    protected TextField timeField;
    protected ChoiceField eventTypeField;
    protected TextField argsField;
    protected Button deleteButton;
    private EventListPanel parent;
    private int rowIndex;

    public EventRow(EventListPanel parent, int rowIndex) {
        this.parent = parent;
        this.rowIndex = rowIndex;
    }

    public void actionPerformed(ActionEvent ev) {
        deleteThisEvent();
    }

    protected void deleteThisEvent() {
        parent.remove(timeField);
        parent.remove(eventTypeField);
        parent.remove(argsField);
        parent.remove(deleteButton);
        parent.removeEventRow(rowIndex);
    }

    //Decrement the gridy constraint value for all our components
    protected void decrementGridBagConstraintGridy(GridBagLayout gbl) {
        decrementGridBagConstraintGridy(gbl,timeField);
        decrementGridBagConstraintGridy(gbl,eventTypeField);
        decrementGridBagConstraintGridy(gbl,argsField);
        decrementGridBagConstraintGridy(gbl,deleteButton);
    }

    //Decrement the gridy constraint value for a component
    private void decrementGridBagConstraintGridy(
            GridBagLayout gbl, Component comp) {
        GridBagConstraints gbc = gbl.getConstraints(comp);
        gbc.gridy--;
        gbl.setConstraints(comp,gbc);
    }

    public SimEvent getSimEvent() {
        String timeStr = timeField.getText().trim();
        String args = argsField.getText().trim();
        if (timeStr.equals("") && args.equals("")) {
            return SimEvent.createSimEvent(0.0,SimEvent.TYPE_NOP,args);
        }
        double time = Double.parseDouble(timeStr);
        int type = SimEvent.TYPE_BREAK;   //TODO - get from choice
        return SimEvent.createSimEvent(time,type,args);
    }
}

/** The panel that has the list of all of our events.
 */
class EventListPanel extends Panel {
    private GridBagLayout eventGbl;
    private GridBagConstraints eventGbc;
    private EventPanel parent;
    private EventRow[] eventRows;

    public EventListPanel(EventPanel parent) {
        super();
        this.parent = parent;

        eventGbl = new GridBagLayout();
        this.setLayout(eventGbl);
        eventGbc = new GridBagConstraints();
        eventGbc.anchor = GridBagConstraints.NORTH;

        eventGbc.gridx = 0;
        eventGbc.gridy = 0;
        eventGbc.gridwidth = 1;
        eventGbc.weightx = 1;
        eventGbc.weighty = 0;
        addTitleRow(eventGbl,eventGbc);

        eventGbc.fill = GridBagConstraints.BOTH;
        eventGbc.gridy = 100;
        eventGbc.weighty = 1;
        Label fillerLabel = new Label("");
        eventGbl.setConstraints(fillerLabel,eventGbc);
        add(fillerLabel);

        eventGbc.fill = GridBagConstraints.HORIZONTAL;
        eventGbc.gridy = 1;
        eventGbc.gridx = 0;
        eventGbc.weighty = 0;
        eventRows = new EventRow[0];
        addEventRow();
    }

    protected void removeEventRow(int rowNum) {
        for (int e=rowNum; e<eventRows.length; e++) {
            //decrement the gridbagconstraint gridy of each field in the row
            EventRow row = eventRows[e];
            row.decrementGridBagConstraintGridy(eventGbl);
        }
        EventRow[] newRows = new EventRow[eventRows.length - 1];
        if (eventRows.length-1>0) {
            System.arraycopy(eventRows,0,newRows,0,rowNum);
            System.arraycopy(eventRows,rowNum+1,
                    newRows,rowNum,eventRows.length-rowNum-1);
        }
        eventRows = newRows;
        parent.fixLayout();
    }

    protected void addEventRow() {
        EventRow eRow = addEventRow(eventGbl,eventGbc,eventRows.length);
        EventRow[] newRows = new EventRow[eventRows.length+1];
        System.arraycopy(eventRows,0,newRows,0,eventRows.length);
        newRows[eventRows.length] = eRow;
        eventRows = newRows;
        parent.fixLayout();
    }

    private void addTitleRow(GridBagLayout gbl, GridBagConstraints gbc) {
        Label timeLabel = new Label("Time");
        Label eventTypeLabel = new Label("Event Type");
        Label argsLabel = new Label("Event Parameters");

        gbl.setConstraints(timeLabel,gbc);
        add(timeLabel);
        gbc.gridx++;
        gbl.setConstraints(eventTypeLabel,gbc);
        add(eventTypeLabel);
        gbc.gridx++;
        gbl.setConstraints(argsLabel,gbc);
        add(argsLabel);
        gbc.gridx++;
    }

    String[] eventTypeChoices = { "Break" };
    private EventRow addEventRow(GridBagLayout gbl, GridBagConstraints gbc,
            int rowIndex) {
        EventRow eRow = new EventRow(this,rowIndex);
        eRow.timeField = new TextField(10);
        eRow.eventTypeField = new ChoiceField();
        for (int i=0; i<eventTypeChoices.length; i++) {
            eRow.eventTypeField.add(eventTypeChoices[i]);
        }
        eRow.argsField = new TextField(20);

        gbl.setConstraints(eRow.timeField,gbc);
        add(eRow.timeField);
        gbc.gridx++;
        gbl.setConstraints(eRow.eventTypeField,gbc);
        add(eRow.eventTypeField);
        gbc.gridx++;
        gbl.setConstraints(eRow.argsField,gbc);
        add(eRow.argsField);
        gbc.gridx++;

        eRow.deleteButton = new Button("Delete");
        eRow.deleteButton.addActionListener(eRow);
        gbl.setConstraints(eRow.deleteButton,gbc);
        add(eRow.deleteButton);

        gbc.gridx = 0;
        gbc.gridy++;

        return eRow;
    }

    protected SimEvent[] collectSimEvents() {
        SimEvent[] simEvents = new SimEvent[eventRows.length];
        for (int i=0; i<simEvents.length; i++) {
            simEvents[i] = eventRows[i].getSimEvent();
        }
        Arrays.sort(simEvents,new SimEvent.TimeComparator());
        return simEvents;
    }
}

//end
