/* LineDataPanel.java - show the line data for the simulation
 *
 * Jim McBeath, November 19, 2007
 */

package net.jimmc.spelsim;

import java.awt.BorderLayout;
import java.awt.Checkbox;
import java.awt.Container;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Label;
import java.awt.Panel;
import java.awt.ScrollPane;
import java.util.HashMap;
import java.util.Map;

public class LineDataPanel extends Panel {
    private TabPanel tabPanel;
    private LineCanvas lineCanvas;
    private LineChoicePanel lineChoicePanel;
    private SpelCanvas spelCanvas;

    public final static String[] itemNames = {
            "Position (Polar)",
            "Position (Rectangular)",
            "Position Ideal (Polar)",
            "Position Ideal (Rectangular)",
            "Position Drift (Polar)",
            "Position Drift (Rectangular)",

            "Velocity (Polar)",
            "Velocity (Rectangular)",
            "Velocity Ideal (Polar)",
            "Velocity Ideal (Rectangular)",
            "Velocity Drift (Polar)",
            "Velocity Drift (Rectangular)",

            "Acceleration (Polar)",
            "Acceleration (Rectangular)",
            "Acceleration Ideal (Polar)",
            "Acceleration Ideal (Rectangular)",
            "Acceleration Drift (Polar)",
            "Acceleration Drift (Rectangular)",

            "Stress",
            "Broken"
    };
    public static final String[] preselectedItemNames = {
            "Position Drift (Polar)",
            "Stress",
    };

    public LineDataPanel(SpelCanvas canvas) {
        spelCanvas = canvas;
        tabPanel = new TabPanel() {
            public void showTab(String name) {
                super.showTab(name);
                updateLineCanvas(spelCanvas);
                    //ensure our lines are drawn when we switch from the
                    //setup tab to the chart tab
            }
        };
        lineChoicePanel = new LineChoicePanel();
        ScrollPane lineChoiceScrollPane = new ScrollPane();
        lineChoiceScrollPane.add(lineChoicePanel);
        lineCanvas = new LineCanvas();
        tabPanel.addControlComponent(new Label("Line Charts: "));
        tabPanel.addTab(lineChoiceScrollPane,"Setup");
        String chartTabName = "Chart";
        tabPanel.addTab(lineCanvas,chartTabName);
        tabPanel.showTab(chartTabName);

        setLayout(new BorderLayout());
        add(tabPanel,BorderLayout.CENTER);

        canvas.addSimListener(new ISimAdapter() {
            public void simStepped(SpelCanvas c, boolean didDraw) {
                if (!didDraw)
                    return;
                updateLineCanvas(c);
            }
        });
    }

    public void updateLineCanvas(SpelCanvas c) {
      synchronized (lineCanvas) {
        int[] oldDrawingOrder = lineCanvas.getDrawingOrder();
        lineCanvas.clearDataSets();     //clear out the old

        if (lineChoicePanel.isEnabled("Position (Polar)")) {
            DVector[] drifts = c.getCable().getPositions();
            lineCanvas.addPolarDataSets("Position",drifts);
        }

        if (lineChoicePanel.isEnabled("Position (Rectangular)")) {
            DVector[] drifts = c.getCable().getPositions();
            lineCanvas.addRectangularDataSets("Position",drifts);
        }

        if (lineChoicePanel.isEnabled("Position Ideal (Polar)")) {
            DVector[] drifts = c.getCable().getIdealPositions();
            lineCanvas.addPolarDataSets("PositionIdeal",drifts);
        }

        if (lineChoicePanel.isEnabled("Position Ideal (Rectangular)")) {
            DVector[] drifts = c.getCable().getIdealPositions();
            lineCanvas.addRectangularDataSets("PositionIdeal",drifts);
        }

        if (lineChoicePanel.isEnabled("Position Drift (Polar)")) {
            DVector[] drifts = c.getCable().getPositionDrifts();
            lineCanvas.addPolarDataSets("PositionDrift",drifts);
        }

        if (lineChoicePanel.isEnabled("Position Drift (Rectangular)")) {
            DVector[] drifts = c.getCable().getPositionDrifts();
            lineCanvas.addRectangularDataSets("PositionDrift",drifts);
        }

        if (lineChoicePanel.isEnabled("Velocity (Polar)")) {
            DVector[] drifts = c.getCable().getVelocities();
            lineCanvas.addPolarDataSets("Velocity",drifts);
        }

        if (lineChoicePanel.isEnabled("Velocity (Rectangular)")) {
            DVector[] drifts = c.getCable().getVelocities();
            lineCanvas.addRectangularDataSets("Velocity",drifts);
        }

        if (lineChoicePanel.isEnabled("Velocity Ideal (Polar)")) {
            DVector[] drifts = c.getCable().getIdealVelocities();
            lineCanvas.addPolarDataSets("VelocityIdeal",drifts);
        }

        if (lineChoicePanel.isEnabled("Velocity Ideal (Rectangular)")) {
            DVector[] drifts = c.getCable().getIdealVelocities();
            lineCanvas.addRectangularDataSets("VelocityIdeal",drifts);
        }

        if (lineChoicePanel.isEnabled("Velocity Drift (Polar)")) {
            DVector[] drifts = c.getCable().getVelocityDrifts();
            lineCanvas.addPolarDataSets("VelocityDrift",drifts);
        }

        if (lineChoicePanel.isEnabled("Velocity Drift (Rectangular)")) {
            DVector[] drifts = c.getCable().getVelocityDrifts();
            lineCanvas.addRectangularDataSets("VelocityDrift",drifts);
        }

        if (lineChoicePanel.isEnabled("Acceleration (Polar)")) {
            DVector[] drifts = c.getCable().getAccelerations();
            lineCanvas.addPolarDataSets("Acceleration",drifts);
        }

        if (lineChoicePanel.isEnabled("Acceleration (Rectangular)")) {
            DVector[] drifts = c.getCable().getAccelerations();
            lineCanvas.addRectangularDataSets("Acceleration",drifts);
        }

        if (lineChoicePanel.isEnabled("Acceleration Ideal (Polar)")) {
            DVector[] drifts = c.getCable().getIdealAccelerations();
            lineCanvas.addPolarDataSets("AccelerationIdeal",drifts);
        }

        if (lineChoicePanel.isEnabled("Acceleration Ideal (Rectangular)")) {
            DVector[] drifts = c.getCable().getIdealAccelerations();
            lineCanvas.addRectangularDataSets("AccelerationIdeal",drifts);
        }

        if (lineChoicePanel.isEnabled("Acceleration Drift (Polar)")) {
            DVector[] drifts = c.getCable().getAccelerationDrifts();
            lineCanvas.addPolarDataSets("AccelerationDrift",drifts);
        }

        if (lineChoicePanel.isEnabled("Acceleration Drift (Rectangular)")) {
            DVector[] drifts = c.getCable().getAccelerationDrifts();
            lineCanvas.addRectangularDataSets("AccelerationDrift",drifts);
        }

        if (lineChoicePanel.isEnabled("Stress")) {
            double[] stresses = c.getCable().getStresses();
            lineCanvas.addDataSet("Stress",stresses);
        }

        if (lineChoicePanel.isEnabled("Broken")) {
            boolean[] broken = c.getCable().getBroken();
            lineCanvas.addDataSet("Broken",broken);
        }

        if (oldDrawingOrder!=null &&
                (oldDrawingOrder.length==lineCanvas.getDataSetCount())) {
            //If we had a drawing order earlier, keep it.
            //This only works if we have the same number of data set items.
            lineCanvas.setDrawingOrder(oldDrawingOrder);
        }
      }
    }

    /** A class to manage the display list of data set choices. */
    static class LineChoicePanel extends Panel {
        private final static int NUM_PRESELECTED_LINES = 4;
                //the first N lines of the above array are preselected
        private Map itemControls;
        public LineChoicePanel() {
            itemControls = new HashMap();
            this.setLayout(new GridBagLayout());
            GridBagConstraints gbc = new GridBagConstraints();
            for (int i=0; i<itemNames.length; i++) {
                ItemControl c = addItemControl(itemNames[i],gbc);
                itemControls.put(itemNames[i],c);
                c.setSelected(false);
            }

            //Enable the preselected items
            for (int i=0; i<preselectedItemNames.length; i++) {
                String itemName = preselectedItemNames[i];
                ItemControl c = getItemControl(itemName);
                if (c!=null)
                    c.setSelected(true);
            }
        }

        private ItemControl addItemControl(String itemName,
                GridBagConstraints gbc) {
            ItemControl p = new ItemControl(itemName);
            p.addDisplayFields(this,gbc);
            return p;
        }

        /** Given an item name, get the ItemControl for that name. */
        private ItemControl getItemControl(String name) {
            return (ItemControl)itemControls.get(name);
        }

        public boolean isEnabled(String itemName) {
            ItemControl c = getItemControl(itemName);
            if (c==null)
                return false;
            return c.isSelected();
        }
    }

    /** A collection of display fields for a single selectable data item. */
    static class ItemControl {
        private String name;
        private Checkbox checkbox;
        public ItemControl(String name) {
            this.name = name;
        }
        void addDisplayFields(Container p, GridBagConstraints gbc) {
            gbc.gridy++;
            gbc.gridx = 0;
            p.add(new Label(name),gbc);
            gbc.gridx++;
            checkbox = new Checkbox();
            p.add(checkbox,gbc);
        }

        public boolean isSelected() {
            return checkbox.getState();
        }
        public void setSelected(boolean selected) {
            checkbox.setState(selected);
        }
    }
}
