/* MaterialPanel.java
 *
 * Jim McBeath, March 14, 2003
 */

package net.jimmc.spelsim;

class MaterialPanel extends PartPanel {
    final Material material;
    ChoiceField materialChoice;
    DoubleField yieldStrengthField;
    DoubleField densityField;
    DoubleField youngsModulusField;
    DoubleField yieldStrainField;

    public MaterialPanel(SpelControls controls, Material m) {
    	super(controls);
	this.material = m;
	String[] knownMaterials = Material.getMaterialNames();
	String[] materials = addCustom(knownMaterials);
	materialChoice = addChoice("Material",materials,partFont);
	materialChoice.addChoiceListener(new ChoiceListener() {
	    public void choiceSelected(String s) {
	    	selectMaterial(s);
	    }
	});
	yieldStrengthField = addDoubleField("Yield Strength","N/(m^2)");
	yieldStrengthField.addEnterListener(new EnterListener() {
	    public void valueEntered(double d) {
	        material.setYieldStrength(d);
		updateDependentValues();
	    }
	});
	densityField = addDoubleField("Density","kg/(m^3)");
	densityField.addEnterListener(new EnterListener() {
	    public void valueEntered(double d) {
	        material.setDensity(d);
		updateDependentValues();
	    }
	});
	youngsModulusField = addDoubleField("Young's Modulus","N/(m^2)");
	youngsModulusField.addEnterListener(new EnterListener() {
	    public void valueEntered(double d) {
	        material.setYoungsModulus(d);
		updateDependentValues();
	    }
	});
	yieldStrainField = addDoubleField("Yield Strain","",readOnlyFont);
	yieldStrainField.setEditable(false);

	materialChoice.select("Carbon Nanotubes");
        selectMaterial(materialChoice.getSelectedItem());
    }

    void selectMaterial(String s) {
        boolean isCustom = s.equals("custom");
	yieldStrengthField.setEditable(isCustom);
	densityField.setEditable(isCustom);
	youngsModulusField.setEditable(isCustom);
	if (isCustom)
	    return;
	Material m = Material.getMaterial(s);
	yieldStrengthField.setValue(m.getYieldStrength());
	densityField.setValue(m.getDensity());
	youngsModulusField.setValue(m.getYoungsModulus());
	material.setYieldStrength(yieldStrengthField.getDoubleValue());
	material.setDensity(densityField.getDoubleValue());
	material.setYoungsModulus(youngsModulusField.getDoubleValue());
	updateDependentValues();
    }

    void updateDependentValues() {
    	yieldStrainField.setValue(material.getYieldStrain());
        controls.updateCable();
    }
}

//end
