/* PartPanel.java
 *
 * Jim McBeath, March 14, 2003
 */

package net.jimmc.spelsim;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Label;
import java.awt.Panel;
import java.awt.TextField;

class PartPanel extends Panel {
    SpelControls controls;
    protected GridBagLayout gb;
    protected GridBagConstraints gbc;
    protected Font inputFont;
    protected Font partFont;
    protected Font readOnlyFont;
    private Label leftLabel;    //the most recently created left label
    private Label rightLabel;    //the most recently created right label

    public PartPanel(SpelControls controls) {
    	this.controls = controls;
	gb = new GridBagLayout();
	gbc = new GridBagConstraints();
	gbc.gridy = 0;
	inputFont = new Font("Monospaced",Font.PLAIN,12);
	partFont = new Font("SansSerif",Font.BOLD,12);
	readOnlyFont = new Font("SansSerif",Font.ITALIC,12);
	setLayout(gb);
    }

    public Font getPartFont() {
        return partFont;
    }

    /** Get the most recently created left label. */
    public Label getLastLeftLabel() {
        return leftLabel;
    }

    /** Get the most recently created right label. */
    public Label getLastRightLabel() {
        return rightLabel;
    }

    /** Add a field with some labels. */
    private void addField(String label, String unitsLabel, Font labelFont,
            Component field) {
        gbc.gridwidth = 1;
	gbc.weighty = 1;
	gbc.anchor = GridBagConstraints.WEST;

	gbc.gridx = 0;
	gbc.weightx = 0;
	leftLabel = new Label(label);
	if (labelFont!=null)
	    leftLabel.setFont(labelFont);
	gb.setConstraints(leftLabel,gbc);
	add(leftLabel);

	gbc.gridx++;
	gbc.weightx = 1;
        if (unitsLabel==null) {
            rightLabel = null;
            gbc.gridwidth = 2;  //if null unitsLabel, make our main field wider
        }
	if (inputFont!=null && (field instanceof TextField)) {
	    field.setFont(inputFont);
	}
	gb.setConstraints(field,gbc);
	add(field);

        if (unitsLabel!=null) {
            gbc.gridx++;
            gbc.weightx = 0;
            rightLabel = new Label(unitsLabel);
            gb.setConstraints(rightLabel,gbc);
            add(rightLabel);
        }

	gbc.gridy++;
    }

    void addLabel(String labelText) {
	Label label = new Label(labelText);
	gbc.gridx = 0;
        gbc.gridwidth = 3;
	gbc.weighty = 0;
	gbc.anchor = GridBagConstraints.WEST;
	gb.setConstraints(label,gbc);
	add(label);
	gbc.gridy++;
    }

    ChoiceField addChoice(String label, String[] items, Font labelFont) {
    	ChoiceField choice = new ChoiceField();
	for (int i=0; i<items.length; i++) {
	    choice.add(items[i]);
	}
        addField(label, null, labelFont, choice);
	return choice;
    }

    TextField addTextField(String label, int width, String unitsLabel) {
        return addTextField(label, width, unitsLabel, null);
    }

    TextField addTextField(
            String label, int width, String unitsLabel, Font labelFont){
	TextField textField = new TextField(width) {
            public void setEditable(boolean editable) {
                super.setEditable(editable);
                setBackground(editable?Color.white:Color.lightGray);
            }
        };
        addField(label, unitsLabel, labelFont, textField);
        return textField;
    }

    IntegerSpinnerField addIntegerSpinnerField(
            String label, String unitsLabel) {
	IntegerSpinnerField textField = new IntegerSpinnerField();
	if (inputFont!=null)
	    textField.setFont(inputFont);
        addField(label, unitsLabel, null, textField);
        return textField;
    }

    IntegerField addIntegerField(String label, String unitsLabel) {
        return addIntegerField(label, unitsLabel, null);
    }

    IntegerField addIntegerField(
            String label, String unitsLabel, Font labelFont){
	IntegerField textField = new IntegerField();
        addField(label, unitsLabel, labelFont, textField);
        return textField;
    }

    DoubleField addDoubleField(String label, String unitsLabel) {
        return addDoubleField(label, unitsLabel, null);
    }

    DoubleField addDoubleField(String label, String unitsLabel, Font labelFont){
	DoubleField textField = new DoubleField();
        addField(label, unitsLabel, labelFont, textField);
        return textField;
    }

    /** Given an array of strings, return a new array one longer than
     * the input, with the word "custom" added to the end.
     */
    public String[] addCustom(String[] src) {
        String[] dest = new String[src.length+1];
	System.arraycopy(src,0,dest,0,src.length);
	dest[src.length] = "custom";
	return dest;
    }
}

//end
