/* PlanetPanel.java
 *
 * Jim McBeath, March 14, 2003
 */

package net.jimmc.spelsim;

class PlanetPanel extends PartPanel {
    final Planet planet;
    ChoiceField planetChoice;
    DoubleField massField;
    DoubleField radiusField;
    DoubleField periodField;
    DoubleField gravityField;
    DoubleField densityField;
    DoubleField syncAltField;

    public PlanetPanel(SpelControls controls, Planet p) {
    	super(controls);
	this.planet = p;
	String[] knownPlanets = Planet.getPlanetNames();
	String[] planets = addCustom(knownPlanets);
	planetChoice = addChoice("Planet",planets,partFont);
	planetChoice.addChoiceListener(new ChoiceListener() {
	    public void choiceSelected(String s) {
	    	selectPlanet(s);
	    }
	});
	massField = addDoubleField("Mass","kg");
	massField.addEnterListener(new EnterListener() {
	    public void valueEntered(double d) {
	        planet.setMass(d);
		updateDependentValues();
	    }
	});
	radiusField = addDoubleField("Radius","m");
	radiusField.addEnterListener(new EnterListener() {
	    public void valueEntered(double d) {
	        planet.setRadius(d);
		updateDependentValues();
	    }
	});
	periodField = addDoubleField("Rotation Period","s");
	periodField.addEnterListener(new EnterListener() {
	    public void valueEntered(double d) {
	        planet.setPeriod(d);
		updateDependentValues();
	    }
	});
	gravityField = addDoubleField("Surface Gravity","m/(s^2)",readOnlyFont);
	gravityField.setEditable(false);
	densityField = addDoubleField("Density","kg/(m^3)",readOnlyFont);
	densityField.setEditable(false);
	syncAltField = addDoubleField("Synchronous Altitude","m",readOnlyFont);
	syncAltField.setEditable(false);

	planetChoice.select("Earth");
        selectPlanet(planetChoice.getSelectedItem());
    }

    void selectPlanet(String s) {
        boolean isCustom = s.equals("custom");
	massField.setEditable(isCustom);
	radiusField.setEditable(isCustom);
	periodField.setEditable(isCustom);
	if (isCustom)
	    return;
	Planet p = Planet.getPlanet(s);
	massField.setValue(p.getMass());
	radiusField.setValue(p.getRadius());
	periodField.setValue(p.getPeriod());
	planet.setMass(massField.getDoubleValue());
	planet.setRadius(radiusField.getDoubleValue());
	planet.setPeriod(periodField.getDoubleValue());
	updateDependentValues();
    }

    void updateDependentValues() {
	gravityField.setValue(planet.getGravity());
	densityField.setValue(planet.getDensity());
	syncAltField.setValue(planet.getSynchronousAltitude());
	controls.updateCable();
    }
}

//end
