/* SimEvent.java
 *
 * Jim McBeath, December 19, 2007
 */

package net.jimmc.spelsim;

import java.util.Comparator;

/** An event for the simulator.
 */
public class SimEvent {
    private double time;        //The event should happen at this time
    private int type;           //Event type
        public final static int TYPE_NOP = 0; //do nothing
        public final static int TYPE_BREAK = 1; //cable break at single point
    private boolean done;       //true after this event has been simulated
    private String args;

    //Factory for creating a SimEvent
    public static SimEvent createSimEvent(double time, int type, String args) {
        SimEvent ev = new SimEvent(time,type,args);
        return ev;
    }

    private SimEvent(double time, int type, String args) {
        this.time = time;
        this.type = type;
        this.args = args;
    }

    public double getTime() { return time; }
    public int getType() { return type; }
    public String getArgs() { return args; }

    //Get the one argument as an integer when that's all there is
    public int getArg0Int() { return Integer.parseInt(args); }

    public boolean isDone() { return done; }
    public void setDone() { done = true; }
    public void setDone(boolean d) { done = d; }

    public boolean equals(Object thatObj) {
        if (!(thatObj instanceof SimEvent))
            return false;
        SimEvent that = (SimEvent)thatObj;
        if (time!=that.time)
            return false;
        if (type!=that.type)
            return false;
        if (args!=that.args || (args!=null && !args.equals(that.args)))
            return false;
        return true;
    }
    public int hashCode() {
        int h = 0;
        if (args!=null)
            h ^= args.hashCode();
        long timeBits = Double.doubleToLongBits(time);
        h ^= timeBits & 0xFFFFFFFFL;
        h ^= timeBits>>32;
        h ^= type<<10;
        return h;
    }

    public static class TimeComparator implements Comparator {
        public int compare(Object a, Object b) {
            SimEvent sa = (SimEvent)a;
            SimEvent sb = (SimEvent)b;
            if (sa.time>sb.time)
                return 1;
            else if (sa.time<sb.time)
                return -1;
            else
                return 0;
        }
    }
}
