/* SimPanel.java
 *
 * Jim McBeath, April 14, 2003
 */

package net.jimmc.spelsim;

import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Label;
import java.awt.Panel;
import java.awt.TextArea;

class SimPanel extends Panel {
    LineDataPanel lineDataPanel;
    public SimPanel(SpelControls controls, SpelCanvas canvas) {
        super();

        SimControlPanel simControlPanel = new SimControlPanel(controls, canvas);
        Font partFont = simControlPanel.getPartFont();
	Panel buttonPanel = new SimButtonPanel(canvas, partFont);
        Panel simStatusPanel = new SimStatusPanel(canvas);
        lineDataPanel = new LineDataPanel(canvas);

        TabPanel simInfoPanel = new TabPanel();
        String parametersTabName = "Parameters";
        simInfoPanel.addTab(simControlPanel,parametersTabName);
        simInfoPanel.addTab(simStatusPanel,"Status");
        simInfoPanel.showTab(parametersTabName);

        GridBagLayout gbl = new GridBagLayout();
        this.setLayout(gbl);
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.anchor = GridBagConstraints.NORTH;

        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.gridwidth = 2;
        gbc.weightx = 0;
        gbc.weighty = 0;
        this.add(buttonPanel, gbc);

        gbc.gridwidth = 1;
        gbc.gridheight = 1;
        gbc.weightx = 1;
        gbc.weighty = 1;
        gbc.anchor = GridBagConstraints.NORTHWEST;
        gbc.fill = GridBagConstraints.BOTH;

        gbc.gridy = 1;
        gbc.weighty = 0;
        this.add(simInfoPanel, gbc);

        gbc.gridy = 3;
        gbc.weighty = 3;
        this.add(lineDataPanel,gbc);

        gbc.gridx = 1;
        gbc.gridy = 1;
        gbc.gridheight = 3;
        gbc.weightx = 3;
        gbc.fill = GridBagConstraints.BOTH;
        this.add(canvas, gbc);
    }
}

class SimButtonPanel extends Panel {
    SpelCanvas canvas;

    public SimButtonPanel(SpelCanvas canvas, Font partFont) {
        super();
        this.canvas = canvas;
        this.setLayout(new FlowLayout());
        Label label = new Label("Simulation");
        label.setFont(partFont);
        this.add(label);

        Button runButton = new Button("Run");
        runButton.addActionListener(new RunAction());
        this.add(runButton);

        Button stopButton = new Button("Stop");
        stopButton.addActionListener(new StopAction());
        this.add(stopButton);

        Button stepButton = new Button("Step");
        stepButton.addActionListener(new StepAction());
        this.add(stepButton);

        Button resetButton = new Button("Reset");
        resetButton.addActionListener(new ResetAction());
        this.add(resetButton);
    }

    class RunAction implements ActionListener {
    	public void actionPerformed(ActionEvent ev) {
	    canvas.start();
	}
    }

    class StopAction implements ActionListener {
    	public void actionPerformed(ActionEvent ev) {
	    canvas.stop();
	}
    }

    class StepAction implements ActionListener {
    	public void actionPerformed(ActionEvent ev) {
	    canvas.step();
	}
    }

    class ResetAction implements ActionListener {
    	public void actionPerformed(ActionEvent ev) {
	    canvas.reset();
	}
    }
}

class SimControlPanel extends PartPanel {
    final SpelCanvas canvas;
    ChoiceField perspectiveChoice;
    ChoiceField scaleChoice;
    ChoiceField debugChoice;
    ChoiceField algorithmChoice;
    DoubleField simStepSizeField;
    DoubleField ticksPerSecondField;
    DoubleField simSpeedField;
    DoubleField dampingFactorField;
    DoubleField cwDeratingField;
    DoubleField groundDragField;
    DoubleField maxRefreshRateField;

    public SimControlPanel(SpelControls controls, SpelCanvas c) {
    	super(controls);
	this.canvas = c;

	simStepSizeField = addDoubleField("Sim Step Size","sSim");
	simStepSizeField.addEnterListener(new EnterListener() {
	    public void valueEntered(double d) {
	        canvas.setSimStepSize(d);
                simSpeedField.setValue(canvas.getSimulationSpeed());
	    }
	});
	simStepSizeField.setValue(canvas.getSimStepSize());
	simStepSizeField.setBackground(Color.white);

	ticksPerSecondField = addDoubleField("Ticks per second","/sReal");
	ticksPerSecondField.addEnterListener(new EnterListener() {
	    public void valueEntered(double d) {
	        canvas.setTicksPerSecond(d);
                simSpeedField.setValue(canvas.getSimulationSpeed());
	    }
	});
	ticksPerSecondField.setValue(canvas.getTicksPerSecond());
	ticksPerSecondField.setBackground(Color.white);

	simSpeedField = addDoubleField("Simulation Speed","sSim/sReal");
	simSpeedField.setValue(canvas.getSimulationSpeed());
	//simSpeedField.setBackground(Color.white);
        simSpeedField.setEditable(false);

	dampingFactorField = addDoubleField("Damping Factor","*dV");
	dampingFactorField.addEnterListener(new EnterListener() {
	    public void valueEntered(double d) {
	        canvas.setDampingFactor(d);
	    }
	});
	dampingFactorField.setValue(canvas.getDampingFactor());
	dampingFactorField.setBackground(Color.white);

	cwDeratingField = addDoubleField("CW Derating","");
	cwDeratingField.addEnterListener(new EnterListener() {
	    public void valueEntered(double d) {
	        canvas.setCwDerating(d);
	    }
	});
	cwDeratingField.setValue(canvas.getCwDerating());
	cwDeratingField.setBackground(Color.white);

	groundDragField = addDoubleField("Ground Drag","");
	groundDragField.addEnterListener(new EnterListener() {
	    public void valueEntered(double d) {
	        canvas.setGroundDrag(d);
	    }
	});
	groundDragField.setValue(canvas.getGroundDrag());
	groundDragField.setBackground(Color.white);

	maxRefreshRateField = addDoubleField("Max Refresh Rate","/sReal");
	maxRefreshRateField.addEnterListener(new EnterListener() {
	    public void valueEntered(double d) {
	        canvas.setMaxRefreshRate(d);
	    }
	});
	maxRefreshRateField.setValue(canvas.getMaxRefreshRate());
	maxRefreshRateField.setBackground(Color.white);

        final String[] scaleChoices = { "Fixed", "Bounding", "Planet" };
	scaleChoice = addChoice("View Scale",scaleChoices,null);
	scaleChoice.addChoiceListener(new ChoiceListener() {
	    public void choiceSelected(String s) {
                for (int x=0; x<scaleChoices.length; x++) {
                    if (s.equals(scaleChoices[x])) {
                        canvas.setViewScale(x);
                        break;
                    }
                }
                //If no valid match found, silently ignore it.
                //We should always find a match.
	    }
	});

	String[] perspectives = { "Sidereal", "Planet" };
	perspectiveChoice = addChoice("Perspective",perspectives,null);
	perspectiveChoice.addChoiceListener(new ChoiceListener() {
	    public void choiceSelected(String s) {
	    	canvas.setPerspective(s.equals("Planet"));
	    }
	});

	final String[] algorithmValues =
                { "Forward", "Fwd Circular", "Trapezoidal" };
	algorithmChoice = addChoice("Algorithm",algorithmValues,null);
	algorithmChoice.addChoiceListener(new ChoiceListener() {
	    public void choiceSelected(String s) {
                for (int x=0; x<algorithmValues.length; x++) {
                    if (s.equals(algorithmValues[x])) {
                        canvas.setAlgorithm(x);
                        break;
                    }
                }
                //If no valid match found, silently ignore it.
                //We should always find a match.
	    }
	});

	String[] debugValues = { "Silent", "Print" };
	debugChoice = addChoice("Debug Output",debugValues,null);
	debugChoice.addChoiceListener(new ChoiceListener() {
	    public void choiceSelected(String s) {
	    	canvas.setDebugOutput(s.equals("Print"));
	    }
	});
    }
}

class SimStatusPanel extends Panel implements ISimListener {
    private TextArea statusField;
    private StringBuffer statusTextBuffer;

    public SimStatusPanel(SpelCanvas canvas) {
        GridBagLayout gbl = new GridBagLayout();
        this.setLayout(gbl);
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.anchor = GridBagConstraints.NORTHEAST;
        gbc.fill = GridBagConstraints.BOTH;

        statusField = new TextArea(3,20);

        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.weightx = 1;
        gbc.weighty = 1;
        this.add(statusField, gbc);

        canvas.addSimListener(this);
        simStepped(canvas,true);
    }

    public void simStepped(SpelCanvas canvas, boolean didDraw) {
        if (!didDraw)
            return;     //we only care about screen updates
        String s = canvas.getStatusString();
        statusField.setText(s);
    }
}

//end
