/* SpelSim.java - Simulate a space elevator.
 *
 * Jim McBeath, March 12, 2003
 */

package net.jimmc.spelsim;

import java.applet.Applet;
import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Event;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.Font;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Label;
import java.awt.Panel;

/** A space elevator simulator. */
public class SpelSim extends java.applet.Applet {
    /** True if we are running as an application rather than an applet. */
    boolean isApplication;

    /** Our frame when running as an application. */
    Frame frame;

    /** Our drawing canvas. */
    SpelCanvas canvas;

    /** Our control panel. */
    SpelControls controls;

    /** Our planet. */
    Planet planet;

    /** Our material definition. */
    Material material;

    /** Our cable configuration. */
    Cable cable;

    /** Get our version info from the manifest in the jar file. */
    public String getVersionInfo() {
        Package pkg = getClass().getPackage();
	String impTitle = pkg.getImplementationTitle();
	String impVersion = pkg.getImplementationVersion();
	return impTitle+" "+impVersion;
    }

    public static void main(String[] args) {
        new SpelSim().doMain(args);
    }

    public void doMain(String[] args) {
        processArgs(args);

	//Interactive
	frame = new Frame("SpelSim");
	frame.addWindowListener(new WindowAdapter() {
	    public void windowClosing(WindowEvent ev) {
	        System.exit(0);
	    }
	});
	isApplication = true;
	init();
	frame.add("Center",this);
	frame.setSize(800,600);
	frame.show();
    }

    protected void processArgs(String[] args) {
        boolean didSomething = false;
        for (int i=0; i<args.length; i++) {
	    if (args[i].equals("--help")) {
	        giveHelp();
		didSomething = true;
	    }
	    else if (args[i].equals("--version")) {
	        System.out.println(getVersionInfo());
		didSomething = true;
	    }
	    else {
	        System.out.println("Unknown command line argument "+args[i]);
		System.exit(1);
	    }
	}
	if (didSomething) {
	    System.exit(0);
	}
    }

    void giveHelp() {
        System.out.println("Options: --version   prints current version");
    }

    public void init() {
	planet = new Planet();
	material = new Material();
	cable = new Cable();
	cable.setPlanet(planet);
	cable.setMaterial(material);

	Component titleArea = getTitleArea();
	titleArea.setFont(new Font("SansSerif",Font.BOLD,14));

        canvas = new SpelCanvas(this);
	controls = new SpelControls(this,canvas);
	SimPanel simPanel = new SimPanel(controls,canvas);
        EventPanel eventPanel = new EventPanel(canvas);
        InspectPanel inspectPanel = new InspectPanel(cable,canvas);

	GridBagLayout gb = new GridBagLayout();
	GridBagConstraints gbc = new GridBagConstraints();
	setLayout(gb);

	gbc.gridx = 0;
	gbc.gridy = 0;
	gbc.weightx = 0;
	gb.setConstraints(titleArea,gbc);
	add(titleArea);

        TabPanel tabPanel = new TabPanel();
        tabPanel.addTab(controls,"Cable");
        tabPanel.addTab(inspectPanel,"Elements");
        tabPanel.addTab(eventPanel,"Events");
        tabPanel.addTab(simPanel,"Simulate");

	gbc.gridy = 1;
	gbc.weighty = 1.0;
	gbc.fill = GridBagConstraints.BOTH;
	gb.setConstraints(tabPanel,gbc);
	add(tabPanel);
    }

    protected Component getTitleArea() {
        Panel p = new Panel();
	Label titleArea = new Label(getVersionInfo());
	p.add(titleArea);
	return p;
    }

    public void start() {
        /* don't start automatically, wait for user to tell it to start...
	if (canvas!=null)
	    canvas.start();
        */
    }

    public void stop() {
	if (canvas!=null)
	    canvas.stop();
    }

    public String getAppletInfo() {
        return getVersionInfo();
    }

    public Planet getPlanet() {
        return planet;
    }

    public Material getMaterial() {
        return material;
    }

    public Cable getCable() {
        return cable;
    }
}

//end
