package net.jimmc.spelsim;

import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Color;
import java.awt.Component;
import java.awt.CardLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.FlowLayout;
import java.awt.Panel;
import java.util.HashMap;
import java.util.Map;

public class TabPanel extends Panel {
    private Panel controlPanel;         //contains the selection tabs
    private FlowLayout controlLayout;
    private Panel bodyPanel;            //contains the client's panels
    private CardLayout bodyLayout;
    private Map tabMap;         //map tab names to body components
    private String currentTabName;       //currently selected tab name

    public TabPanel() {
        super();
        tabMap = new HashMap();

        bodyPanel = new Panel();
        bodyLayout = new CardLayout();
        bodyPanel.setLayout(bodyLayout);

        controlPanel = new Panel();
        controlLayout = new FlowLayout();
        controlPanel.setLayout(controlLayout);

        BorderLayout topLayout = new BorderLayout();
        this.setLayout(topLayout);
        this.add(bodyPanel,BorderLayout.CENTER);
        this.add(controlPanel,BorderLayout.NORTH);
    }

    /** Add a component in the tab button bar. */
    public void addControlComponent(Component comp) {
        controlPanel.add(comp);
    }

    /** Add a tab to the body and a button to the control button bar. */
    public void addTab(Component comp, String name) {
        tabMap.put(name,comp);         //keep our own name-to-component map
        bodyPanel.add(comp, name);
        TabButton button = new TabButton(name,name);
        controlPanel.add(button);
    }
    class TabButton extends Button implements ActionListener {
        private String tabName;
        public TabButton(String tabName,String label) {
            super(label);
            this.tabName = tabName;
            addActionListener(this);
        }
        public void actionPerformed(ActionEvent e) {
            showTab(tabName);
        }
    }

    public void showTab(String name) {
        if (name==null)
            return;             //ignore null
        if (name.equals(currentTabName))
            return;             //already selected
        if (currentTabName!=null) {
            Component oldTab = (Component)tabMap.get(currentTabName);
            if (oldTab instanceof TabSelectListener) {
                ((TabSelectListener)oldTab).tabDeselected();
            }
        }
        bodyLayout.show(bodyPanel, name);
        currentTabName = name;
        Component newTab = (Component)tabMap.get(currentTabName);
        if (newTab instanceof TabSelectListener) {
            ((TabSelectListener)newTab).tabSelected();
        }
    }

    //For testing
    public static void main(String[] args) {
        java.awt.Frame frame = new java.awt.Frame("TabPanel");
        frame.addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent ev) {
                System.exit(0);
            }
        });
        TabPanel topPanel = new TabPanel();
        topPanel.addTab(new java.awt.Label("Pane1"),"Pane1");
        topPanel.addTab(new java.awt.Label("Pane2"),"Pane2");

        //BorderLayout frameLayout = new BorderLayout();
        //frameLayout.addLayoutComponent(topPanel,BorderLayout.CENTER);
        frame.add(topPanel);

        frame.setSize(500,500);
        frame.show();
    }
}
